@@ -1,5 +1,5 @@
 from abc import ABCMeta, abstractmethod
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Tuple
 
 from samtranslator.internal.deprecation_control import deprecated
 from samtranslator.metrics.method_decorator import cw_timer
@@ -52,8 +52,11 @@ class PullEventSource(ResourceMacro, metaclass=ABCMeta):
         "FunctionResponseTypes": PropertyType(False, IS_LIST),
         "KafkaBootstrapServers": PropertyType(False, IS_LIST),
         "FilterCriteria": PropertyType(False, IS_DICT),
+        "KmsKeyArn": PassThroughProperty(False),
         "ConsumerGroupId": PropertyType(False, IS_STR),
         "ScalingConfig": PropertyType(False, IS_DICT),
+        "ProvisionedPollerConfig": PropertyType(False, IS_DICT),
+        "MetricsConfig": PropertyType(False, IS_DICT),
     }
 
     BatchSize: Optional[Intrinsicable[int]]
@@ -74,8 +77,11 @@ class PullEventSource(ResourceMacro, metaclass=ABCMeta):
     FunctionResponseTypes: Optional[List[Any]]
     KafkaBootstrapServers: Optional[List[Any]]
     FilterCriteria: Optional[Dict[str, Any]]
+    KmsKeyArn: Optional[Intrinsicable[str]]
     ConsumerGroupId: Optional[Intrinsicable[str]]
     ScalingConfig: Optional[Dict[str, Any]]
+    ProvisionedPollerConfig: Optional[Dict[str, Any]]
+    MetricsConfig: Optional[Dict[str, Any]]
 
     @abstractmethod
     def get_policy_arn(self) -> Optional[str]:
@@ -98,7 +104,7 @@ class PullEventSource(ResourceMacro, metaclass=ABCMeta):
         return
 
     @cw_timer(prefix=FUNCTION_EVETSOURCE_METRIC_PREFIX)
-    def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def] # noqa: too-many-branches
+    def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def] # noqa: PLR0912, PLR0915
         """Returns the Lambda EventSourceMapping to which this pull event corresponds. Adds the appropriate managed
         policy to the function's execution role, if such a role is provided.
 
@@ -141,7 +147,10 @@ class PullEventSource(ResourceMacro, metaclass=ABCMeta):
         lambda_eventsourcemapping.TumblingWindowInSeconds = self.TumblingWindowInSeconds
         lambda_eventsourcemapping.FunctionResponseTypes = self.FunctionResponseTypes
         lambda_eventsourcemapping.FilterCriteria = self.FilterCriteria
+        lambda_eventsourcemapping.KmsKeyArn = self.KmsKeyArn
         lambda_eventsourcemapping.ScalingConfig = self.ScalingConfig
+        lambda_eventsourcemapping.ProvisionedPollerConfig = self.ProvisionedPollerConfig
+        lambda_eventsourcemapping.MetricsConfig = self.MetricsConfig
         self._validate_filter_criteria()
 
         if self.KafkaBootstrapServers:
@@ -172,13 +181,15 @@ class PullEventSource(ResourceMacro, metaclass=ABCMeta):
             # `Type` property is for sam to attach the right policies
             destination_type = on_failure.get("Type")
 
-            # SAM attaches the policies for SQS or SNS only if 'Type' is given
+            # SAM attaches the policies for SQS, SNS or S3 only if 'Type' is given
             if destination_type:
                 # delete this field as its used internally for SAM to determine the policy
                 del on_failure["Type"]
-                # the values 'SQS' and 'SNS' are allowed. No intrinsics are allowed
-                if destination_type not in ["SQS", "SNS"]:
-                    raise InvalidEventException(self.logical_id, "The only valid values for 'Type' are 'SQS' and 'SNS'")
+                # the values 'SQS', 'SNS', and 'S3' are allowed. No intrinsics are allowed
+                if destination_type not in ["SQS", "SNS", "S3"]:
+                    raise InvalidEventException(
+                        self.logical_id, "The only valid values for 'Type' are 'SQS', 'SNS', and 'S3'"
+                    )
                 if destination_type == "SQS":
                     queue_arn = on_failure.get("Destination")
                     destination_config_policy = IAMRolePolicies().sqs_send_message_role_policy(
@@ -189,6 +200,11 @@ class PullEventSource(ResourceMacro, metaclass=ABCMeta):
                     destination_config_policy = IAMRolePolicies().sns_publish_role_policy(
                         sns_topic_arn, self.logical_id
                     )
+                elif destination_type == "S3":
+                    s3_arn = on_failure.get("Destination")
+                    destination_config_policy = IAMRolePolicies().s3_send_event_payload_role_policy(
+                        s3_arn, self.logical_id
+                    )
 
             lambda_eventsourcemapping.DestinationConfig = self.DestinationConfig
 
@@ -226,7 +242,7 @@ class PullEventSource(ResourceMacro, metaclass=ABCMeta):
             if role.Policies and destination_config_policy not in role.Policies:
                 policy_document = destination_config_policy.get("PolicyDocument")
                 # do not add the policy if the same policy document is already present
-                if policy_document not in [d["PolicyDocument"] for d in role.Policies]:
+                if policy_document not in [d.get("PolicyDocument", {}) for d in role.Policies]:
                     role.Policies.append(destination_config_policy)
 
     def _validate_filter_criteria(self) -> None:
@@ -520,7 +536,6 @@ class SelfManagedKafka(PullEventSource):
         "SASL_SCRAM_512_AUTH",
         "BASIC_AUTH",
         "CLIENT_CERTIFICATE_TLS_AUTH",
-        "SERVER_ROOT_CA_CERTIFICATE",
     ]
 
     def get_event_source_arn(self) -> Optional[PassThrough]:
@@ -558,11 +573,15 @@ class SelfManagedKafka(PullEventSource):
 
     def generate_policy_document(self, source_access_configurations: List[Any]):  # type: ignore[no-untyped-def]
         statements = []
-        authentication_uri, has_vpc_config = self.get_secret_key(source_access_configurations)
+        authentication_uri, authentication_uri_2, has_vpc_config = self.get_secret_key(source_access_configurations)
         if authentication_uri:
             secret_manager = self.get_secret_manager_secret(authentication_uri)  # type: ignore[no-untyped-call]
             statements.append(secret_manager)
 
+        if authentication_uri_2:
+            secret_manager = self.get_secret_manager_secret(authentication_uri)  # type: ignore[no-untyped-call]
+            statements.append(secret_manager)
+
         if has_vpc_config:
             vpc_permissions = self.get_vpc_permission()
             statements.append(vpc_permissions)
@@ -580,10 +599,11 @@ class SelfManagedKafka(PullEventSource):
             "PolicyName": "SelfManagedKafkaExecutionRolePolicy",
         }
 
-    def get_secret_key(self, source_access_configurations: List[Any]):  # type: ignore[no-untyped-def]
+    def get_secret_key(self, source_access_configurations: List[Any]) -> Tuple[Optional[str], Optional[str], bool]:
         authentication_uri = None
         has_vpc_subnet = False
         has_vpc_security_group = False
+        authentication_uri_2 = None
 
         if not isinstance(source_access_configurations, list):
             raise InvalidEventException(
@@ -609,6 +629,10 @@ class SelfManagedKafka(PullEventSource):
                 self.validate_uri(config.get("URI"), "auth mechanism")
                 authentication_uri = config.get("URI")
 
+            elif config.get("Type") == "SERVER_ROOT_CA_CERTIFICATE":
+                self.validate_uri(config.get("URI"), "SERVER_ROOT_CA_CERTIFICATE")
+                authentication_uri_2 = config.get("URI")
+
             else:
                 raise InvalidEventException(
                     self.relative_id,
@@ -620,7 +644,7 @@ class SelfManagedKafka(PullEventSource):
                 self.relative_id,
                 "VPC_SUBNET and VPC_SECURITY_GROUP in SourceAccessConfigurations for SelfManagedKafka must be both provided.",
             )
-        return authentication_uri, (has_vpc_subnet and has_vpc_security_group)
+        return authentication_uri, authentication_uri_2, (has_vpc_subnet and has_vpc_security_group)
 
     def validate_uri(self, uri: Optional[Any], msg: str) -> None:
         if not uri:
@@ -632,9 +656,7 @@ class SelfManagedKafka(PullEventSource):
         if not isinstance(uri, str) and not is_intrinsic(uri):
             raise InvalidEventException(
                 self.relative_id,
-                "Wrong Type for {} URI property specified in SourceAccessConfigurations for self managed kafka event.".format(
-                    msg
-                ),
+                f"Wrong Type for {msg} URI property specified in SourceAccessConfigurations for self managed kafka event.",
             )
 
     def get_secret_manager_secret(self, authentication_uri):  # type: ignore[no-untyped-def]
